/****************************************************************************
 * (C) Tokyo Cosmos Electric, Inc. (TOCOS) - 2013 all rights reserved.
 *
 * Condition to use: (refer to detailed conditions in Japanese)
 *   - The full or part of source code is limited to use for TWE (TOCOS
 *     Wireless Engine) as compiled and flash programmed.
 *   - The full or part of source code is prohibited to distribute without
 *     permission from TOCOS.
 *
 * 利用条件:
 *   - 本ソースコードは、別途ソースコードライセンス記述が無い限り東京コスモス電機が著作権を
 *     保有しています。
 *   - 本ソースコードは、無保証・無サポートです。本ソースコードや生成物を用いたいかなる損害
 *     についても東京コスモス電機は保証致しません。不具合等の報告は歓迎いたします。
 *   - 本ソースコードは、東京コスモス電機が販売する TWE シリーズ上で実行する前提で公開
 *     しています。他のマイコン等への移植・流用は一部であっても出来ません。
 *
 ****************************************************************************/

#ifndef EXPERIMENTAL_H_
#define EXPERIMENTAL_H_

/****************************************************************************/
/***        実験的な実装定義                                              ***/
/****************************************************************************/

/** @ingroup MASTER
 * I2C LCD モジュールテスト用のコード
 *   本定義が実装されると、I2C 接続の LCD デバイスにテスト文字を表示する
 *   - IO入力を受け取った時そのボタンに応じたメッセージを表示
 *   - インタラクティブモードで 1-4 に対応してメッセージを表示
 */
#undef USE_I2C_LCD_TEST_CODE
#ifdef USE_I2C_LCD_TEST_CODE
#warning "EXPERIMENTAL: USE_I2C_LCD_TEST_CODE"
const char astrLcdMsgs[4][2][16] = {
	{	"TEST1 67812345678", "ABCDefghIJKLmnop"},
	{	"TEST2 67812345678", "ABCDefghIJKLmnop"},
	{	"TEST3 67812345678", "ABCDefghIJKLmnop"},
	{	"TEST4 67812345678", "ABCDefghIJKLmnop"},
};
#endif

/** @ingroup MASTER
 * Sleep 中に IO の出力状態を設定する
 *   DO0,1を Sleep 中は Hi, 稼働中は Lo
 */
#undef SET_DO_ON_SLEEP
#ifdef SET_DO_ON_SLEEP
#warning "EXPERIMENTAL: SET_DO_ON_SLEEP"
static uint8 u8SleepControl = 0; //!< 0:事前Sleep, 1:通常
#define PRE_SLEEP_ms 50
#endif

/** @ingroup MASTER
 * BROWNOUT 検出時にリセットする。
 */
#undef USE_BROWN_OUT_DETECT
#ifdef USE_BROWN_OUT_DETECT
# warning "#define USE_BROWN_OUT_DETECT"
#endif

/** @ingroup MASTER
 * 電源投入時に一定時間ポートをONにする
 */
#undef USE_INITIAL_PORT_ON //!< 起動時にポートをONにする
#ifdef USE_INITIAL_PORT_ON
# warning "#define USE_INITIAL_PORT_ON"
#endif

/** @ingroup MASTER
 * I2Cポートを入力として利用し、かつチャネル切り替えに使用する
 *
 * 両方のピンが HI の場合は、インタラクティブモードの設定となり、
 * それ以外の場合は、以下に定義している CHMASK_1-3 に切り替わる。
 */
#undef USE_I2C_PORT_AS_CHANNEL_SET
#ifdef USE_I2C_PORT_AS_CHANNEL_SET
#define USE_I2C_PORT_AS_OTHER_FUNCTION
#warning "EXPERIMENTAL: USE_I2C_PORT_AS_CHANNEL_SET"

#define CHANNEL_1 12
#define CHMASK_1 (1UL << CHANNEL_1) //!< チャネルマスク（３つまで指定する事が出来る）
#define CHANNEL_2 21
#define CHMASK_2 (1UL << CHANNEL_2) //!< チャネルマスク（３つまで指定する事が出来る）
#define CHANNEL_3 24
#define CHMASK_3 (1UL << CHANNEL_3) //!< チャネルマスク（３つまで指定する事が出来る）

// チャネルのテーブル
const uint32 au32ChMask_Preset[] = { 0UL, CHMASK_1, CHMASK_2, CHMASK_3, };

// ピンの選択 (0-3)
static uint8 u8ChPreSet = 0;

static void vChangeChannelPreset(uint8 u8preset);
#endif

/** @ingroup MASTER
 * I2Cポートを入力として利用し、かつ PWM の特殊制御に使用する
 * (USE_I2C_PORT_AS_CHANNEL_SET と両方定義すると、チャネル切り替えと
 *  PWM 制御パターンを同時に切り替える)
 *
 * 両ポートがHIの場合は、PWMのデューティ比設定
 * 他の設定は、サイン、脈動、パルス
 *
 * ※ PWM の出力を反転したい場合は、cbToCoNet_u8HwInt() 中の PWM 設定部で
 *    u16duty 値を反転させるとよいだろう。
 *
 *    sTimerPWM[i].u16duty = 1024 - sTimerPWM[i].u16duty; // 値を反転
 *    vTimerStart(&sTimerPWM[i]);
 */
#undef USE_I2C_PORT_AS_PWM_SPECIAL

#if defined(USE_I2C_PORT_AS_PWM_SPECIAL)
#define USE_I2C_PORT_AS_OTHER_FUNCTION
#warning "EXPERIMENTAL: USE_I2C_PORT_AS_OTHER_FUNCTION"
uint8 u8PwmSpe_TxMode = 0;
uint8 u8PwmSpe_RxMode = 0;

// タイマー管理のためのカウンター
struct {
	uint16 u16Ct_Active[4]; // Active 期間のカウンタ数 (このときだけ光る）
	uint16 u16Ct_Total[4]; // トータルカウンタ数
	uint16 u16Ct_Now[4]; // 現在のカウント値 (100でループする)
} sPwmSpe_Context;

// サイン波のテーブル
const uint16 u16PwmSpe_SinTbl[101] = { 0, 32, 64, 96, 128, 160, 192, 223, 255,
		286, 316, 347, 377, 407, 436, 465, 493, 521, 549, 576, 602, 628, 653,
		677, 701, 724, 746, 768, 789, 809, 828, 847, 865, 881, 897, 912, 927,
		940, 952, 963, 974, 983, 992, 999, 1006, 1011, 1016, 1019, 1022, 1023,
		1024, 1023, 1022, 1019, 1016, 1011, 1006, 999, 992, 983, 974, 963, 952,
		940, 927, 912, 897, 881, 865, 847, 828, 809, 789, 768, 746, 724, 701,
		677, 653, 628, 602, 576, 549, 521, 493, 465, 436, 407, 377, 347, 316,
		286, 255, 223, 192, 160, 128, 96, 64, 32, 0 };

// ふわっとするテーブル
const uint16 u16PwmSpe_fuwa[101] = { 0, 0, 2, 4, 7, 10, 15, 20, 26, 33, 41, 50,
		59, 69, 80, 92, 105, 118, 133, 148, 164, 181, 198, 217, 236, 256, 277,
		299, 321, 344, 369, 394, 419, 446, 473, 502, 531, 561, 591, 623, 655,
		689, 723, 757, 793, 829, 867, 905, 944, 983, 1024, 1024, 1024, 1024,
		1024, 1024, 1024, 1024, 1024, 1024, 960, 896, 832, 768, 704, 640, 576,
		512, 448, 384, 320, 256, 192, 128, 64, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };

// 可変抵抗の C カーブから B カーブへの変換（意味あるかな？）
const uint16 au16_CtoBcurve[1025] = { 0, 4, 8, 12, 16, 20, 24, 28, 32, 36, 40,
		44, 47, 51, 55, 58, 62, 66, 69, 73, 76, 80, 83, 87, 90, 93, 97, 100,
		103, 106, 110, 113, 116, 119, 122, 126, 129, 132, 135, 138, 141, 144,
		147, 150, 153, 156, 158, 161, 164, 167, 170, 173, 175, 178, 181, 184,
		186, 189, 192, 194, 197, 200, 202, 205, 207, 210, 212, 215, 217, 220,
		222, 225, 227, 230, 232, 235, 237, 239, 242, 244, 247, 249, 251, 254,
		256, 258, 260, 263, 265, 267, 269, 272, 274, 276, 278, 280, 282, 285,
		287, 289, 291, 293, 295, 297, 299, 301, 303, 305, 308, 310, 312, 314,
		316, 318, 320, 321, 323, 325, 327, 329, 331, 333, 335, 337, 339, 341,
		343, 344, 346, 348, 350, 352, 354, 355, 357, 359, 361, 363, 364, 366,
		368, 370, 371, 373, 375, 377, 378, 380, 382, 384, 385, 387, 389, 390,
		392, 394, 395, 397, 399, 400, 402, 403, 405, 407, 408, 410, 411, 413,
		415, 416, 418, 419, 421, 422, 424, 426, 427, 429, 430, 432, 433, 435,
		436, 438, 439, 441, 442, 444, 445, 447, 448, 450, 451, 452, 454, 455,
		457, 458, 460, 461, 462, 464, 465, 467, 468, 469, 471, 472, 474, 475,
		476, 478, 479, 480, 482, 483, 484, 486, 487, 488, 490, 491, 492, 494,
		495, 496, 498, 499, 500, 502, 503, 504, 505, 507, 508, 509, 510, 512,
		513, 514, 515, 517, 518, 519, 520, 522, 523, 524, 525, 527, 528, 529,
		530, 531, 533, 534, 535, 536, 537, 539, 540, 541, 542, 543, 544, 546,
		547, 548, 549, 550, 551, 552, 554, 555, 556, 557, 558, 559, 560, 562,
		563, 564, 565, 566, 567, 568, 569, 570, 572, 573, 574, 575, 576, 577,
		578, 579, 580, 581, 582, 583, 584, 585, 587, 588, 589, 590, 591, 592,
		593, 594, 595, 596, 597, 598, 599, 600, 601, 602, 603, 604, 605, 606,
		607, 608, 609, 610, 611, 612, 613, 614, 615, 616, 617, 618, 619, 620,
		621, 622, 623, 624, 625, 626, 627, 628, 629, 630, 631, 632, 633, 634,
		634, 635, 636, 637, 638, 639, 640, 641, 642, 643, 644, 645, 646, 647,
		647, 648, 649, 650, 651, 652, 653, 654, 655, 656, 657, 657, 658, 659,
		660, 661, 662, 663, 664, 665, 665, 666, 667, 668, 669, 670, 671, 671,
		672, 673, 674, 675, 676, 677, 678, 678, 679, 680, 681, 682, 683, 683,
		684, 685, 686, 687, 688, 688, 689, 690, 691, 692, 693, 693, 694, 695,
		696, 697, 698, 698, 699, 700, 701, 702, 702, 703, 704, 705, 706, 706,
		707, 708, 709, 710, 710, 711, 712, 713, 713, 714, 715, 716, 717, 717,
		718, 719, 720, 721, 721, 722, 723, 724, 724, 725, 726, 727, 727, 728,
		729, 730, 730, 731, 732, 733, 733, 734, 735, 736, 736, 737, 738, 739,
		739, 740, 741, 742, 742, 743, 744, 745, 745, 746, 747, 747, 748, 749,
		750, 750, 751, 752, 752, 753, 754, 755, 755, 756, 757, 757, 758, 759,
		760, 760, 761, 762, 762, 763, 764, 764, 765, 766, 767, 767, 768, 769,
		769, 770, 771, 771, 772, 773, 773, 774, 775, 775, 776, 777, 777, 778,
		779, 780, 780, 781, 782, 782, 783, 784, 784, 785, 786, 786, 787, 787,
		788, 789, 789, 790, 791, 791, 792, 793, 793, 794, 795, 795, 796, 797,
		797, 798, 799, 799, 800, 800, 801, 802, 802, 803, 804, 804, 805, 806,
		806, 807, 807, 808, 809, 809, 810, 811, 811, 812, 812, 813, 814, 814,
		815, 816, 816, 817, 817, 818, 819, 819, 820, 820, 821, 822, 822, 823,
		824, 824, 825, 825, 826, 827, 827, 828, 828, 829, 830, 830, 831, 831,
		832, 833, 833, 834, 834, 835, 835, 836, 837, 837, 838, 838, 839, 840,
		840, 841, 841, 842, 843, 843, 844, 844, 845, 845, 846, 847, 847, 848,
		848, 849, 849, 850, 851, 851, 852, 852, 853, 853, 854, 855, 855, 856,
		856, 857, 857, 858, 858, 859, 860, 860, 861, 861, 862, 862, 863, 863,
		864, 865, 865, 866, 866, 867, 867, 868, 868, 869, 869, 870, 871, 871,
		872, 872, 873, 873, 874, 874, 875, 875, 876, 876, 877, 878, 878, 879,
		879, 880, 880, 881, 881, 882, 882, 883, 883, 884, 884, 885, 885, 886,
		887, 887, 888, 888, 889, 889, 890, 890, 891, 891, 892, 892, 893, 893,
		894, 894, 895, 895, 896, 896, 897, 897, 898, 898, 899, 899, 900, 900,
		901, 901, 902, 902, 903, 903, 904, 904, 905, 905, 906, 906, 907, 907,
		908, 908, 909, 909, 910, 910, 911, 911, 912, 912, 913, 913, 914, 914,
		915, 915, 916, 916, 917, 917, 918, 918, 919, 919, 920, 920, 921, 921,
		922, 922, 923, 923, 924, 924, 925, 925, 926, 926, 926, 927, 927, 928,
		928, 929, 929, 930, 930, 931, 931, 932, 932, 933, 933, 934, 934, 934,
		935, 935, 936, 936, 937, 937, 938, 938, 939, 939, 940, 940, 941, 941,
		941, 942, 942, 943, 943, 944, 944, 945, 945, 946, 946, 946, 947, 947,
		948, 948, 949, 949, 950, 950, 951, 951, 951, 952, 952, 953, 953, 954,
		954, 955, 955, 955, 956, 956, 957, 957, 958, 958, 959, 959, 959, 960,
		960, 961, 961, 962, 962, 963, 963, 963, 964, 964, 965, 965, 966, 966,
		966, 967, 967, 968, 968, 969, 969, 969, 970, 970, 971, 971, 972, 972,
		972, 973, 973, 974, 974, 975, 975, 975, 976, 976, 977, 977, 978, 978,
		978, 979, 979, 980, 980, 981, 981, 981, 982, 982, 983, 983, 983, 984,
		984, 985, 985, 986, 986, 986, 987, 987, 988, 988, 988, 989, 989, 990,
		990, 990, 991, 991, 992, 992, 993, 993, 993, 994, 994, 995, 995, 995,
		996, 996, 997, 997, 997, 998, 998, 999, 999, 999, 1000, 1000, 1001,
		1001, 1001, 1002, 1002, 1003, 1003, 1003, 1004, 1004, 1005, 1005, 1005,
		1006, 1006, 1007, 1007, 1007, 1008, 1008, 1009, 1009, 1009, 1010, 1010,
		1011, 1011, 1011, 1012, 1012, 1012, 1013, 1013, 1014, 1014, 1014, 1015,
		1015, 1016, 1016, 1016, 1017, 1017, 1018, 1018, 1018, 1019, 1019, 1019,
		1020, 1020, 1021, 1021, 1021, 1022, 1022, 1022, 1023, 1023, 1024, 1024 };

// 可変抵抗器の A カーブから B カーブへの変換
const uint16 au16_AtoBcurve[1025] = { 0, 0, 0, 1, 1, 1, 1, 2, 2, 2, 2, 3, 3, 3,
		3, 4, 4, 4, 5, 5, 5, 6, 6, 6, 7, 7, 7, 8, 8, 8, 9, 9, 9, 10, 10, 10, 11,
		11, 12, 12, 12, 13, 13, 13, 14, 14, 15, 15, 15, 16, 16, 17, 17, 17, 18,
		18, 19, 19, 19, 20, 20, 21, 21, 21, 22, 22, 23, 23, 23, 24, 24, 25, 25,
		26, 26, 26, 27, 27, 28, 28, 29, 29, 30, 30, 30, 31, 31, 32, 32, 33, 33,
		34, 34, 34, 35, 35, 36, 36, 37, 37, 38, 38, 39, 39, 40, 40, 41, 41, 41,
		42, 42, 43, 43, 44, 44, 45, 45, 46, 46, 47, 47, 48, 48, 49, 49, 50, 50,
		51, 51, 52, 52, 53, 53, 54, 54, 55, 55, 56, 56, 57, 57, 58, 58, 59, 59,
		60, 60, 61, 61, 62, 63, 63, 64, 64, 65, 65, 66, 66, 67, 67, 68, 68, 69,
		69, 70, 71, 71, 72, 72, 73, 73, 74, 74, 75, 75, 76, 77, 77, 78, 78, 79,
		79, 80, 80, 81, 82, 82, 83, 83, 84, 84, 85, 86, 86, 87, 87, 88, 88, 89,
		90, 90, 91, 91, 92, 92, 93, 94, 94, 95, 95, 96, 97, 97, 98, 98, 99, 100,
		100, 101, 101, 102, 103, 103, 104, 104, 105, 106, 106, 107, 107, 108,
		109, 109, 110, 111, 111, 112, 112, 113, 114, 114, 115, 115, 116, 117,
		117, 118, 119, 119, 120, 120, 121, 122, 122, 123, 124, 124, 125, 126,
		126, 127, 128, 128, 129, 129, 130, 131, 131, 132, 133, 133, 134, 135,
		135, 136, 137, 137, 138, 139, 139, 140, 141, 141, 142, 143, 143, 144,
		145, 145, 146, 147, 147, 148, 149, 149, 150, 151, 151, 152, 153, 153,
		154, 155, 156, 156, 157, 158, 158, 159, 160, 160, 161, 162, 163, 163,
		164, 165, 165, 166, 167, 167, 168, 169, 170, 170, 171, 172, 172, 173,
		174, 175, 175, 176, 177, 177, 178, 179, 180, 180, 181, 182, 183, 183,
		184, 185, 186, 186, 187, 188, 188, 189, 190, 191, 191, 192, 193, 194,
		194, 195, 196, 197, 197, 198, 199, 200, 200, 201, 202, 203, 204, 204,
		205, 206, 207, 207, 208, 209, 210, 210, 211, 212, 213, 214, 214, 215,
		216, 217, 217, 218, 219, 220, 221, 221, 222, 223, 224, 225, 225, 226,
		227, 228, 229, 229, 230, 231, 232, 233, 233, 234, 235, 236, 237, 237,
		238, 239, 240, 241, 242, 242, 243, 244, 245, 246, 246, 247, 248, 249,
		250, 251, 251, 252, 253, 254, 255, 256, 256, 257, 258, 259, 260, 261,
		262, 262, 263, 264, 265, 266, 267, 268, 268, 269, 270, 271, 272, 273,
		274, 274, 275, 276, 277, 278, 279, 280, 281, 281, 282, 283, 284, 285,
		286, 287, 288, 288, 289, 290, 291, 292, 293, 294, 295, 296, 297, 297,
		298, 299, 300, 301, 302, 303, 304, 305, 306, 306, 307, 308, 309, 310,
		311, 312, 313, 314, 315, 316, 317, 318, 318, 319, 320, 321, 322, 323,
		324, 325, 326, 327, 328, 329, 330, 331, 332, 333, 334, 334, 335, 336,
		337, 338, 339, 340, 341, 342, 343, 344, 345, 346, 347, 348, 349, 350,
		351, 352, 353, 354, 355, 356, 357, 358, 359, 360, 361, 362, 363, 364,
		365, 366, 367, 368, 369, 370, 371, 372, 373, 374, 375, 376, 377, 378,
		379, 380, 381, 382, 383, 384, 385, 386, 387, 388, 389, 390, 391, 392,
		393, 394, 395, 396, 397, 398, 399, 400, 401, 402, 404, 405, 406, 407,
		408, 409, 410, 411, 412, 413, 414, 415, 416, 417, 418, 419, 420, 422,
		423, 424, 425, 426, 427, 428, 429, 430, 431, 432, 433, 435, 436, 437,
		438, 439, 440, 441, 442, 443, 444, 446, 447, 448, 449, 450, 451, 452,
		453, 454, 456, 457, 458, 459, 460, 461, 462, 463, 465, 466, 467, 468,
		469, 470, 471, 473, 474, 475, 476, 477, 478, 479, 481, 482, 483, 484,
		485, 486, 488, 489, 490, 491, 492, 493, 495, 496, 497, 498, 499, 501,
		502, 503, 504, 505, 506, 508, 509, 510, 511, 512, 514, 515, 516, 517,
		518, 520, 521, 522, 523, 525, 526, 527, 528, 529, 531, 532, 533, 534,
		536, 537, 538, 539, 540, 542, 543, 544, 545, 547, 548, 549, 550, 552,
		553, 554, 555, 557, 558, 559, 560, 562, 563, 564, 566, 567, 568, 569,
		571, 572, 573, 575, 576, 577, 578, 580, 581, 582, 584, 585, 586, 587,
		589, 590, 591, 593, 594, 595, 597, 598, 599, 601, 602, 603, 605, 606,
		607, 609, 610, 611, 613, 614, 615, 617, 618, 619, 621, 622, 623, 625,
		626, 627, 629, 630, 631, 633, 634, 636, 637, 638, 640, 641, 642, 644,
		645, 647, 648, 649, 651, 652, 653, 655, 656, 658, 659, 660, 662, 663,
		665, 666, 667, 669, 670, 672, 673, 675, 676, 677, 679, 680, 682, 683,
		685, 686, 687, 689, 690, 692, 693, 695, 696, 698, 699, 700, 702, 703,
		705, 706, 708, 709, 711, 712, 714, 715, 717, 718, 720, 721, 723, 724,
		726, 727, 728, 730, 731, 733, 734, 736, 738, 739, 741, 742, 744, 745,
		747, 748, 750, 751, 753, 754, 756, 757, 759, 760, 762, 763, 765, 767,
		768, 770, 771, 773, 774, 776, 777, 779, 781, 782, 784, 785, 787, 788,
		790, 792, 793, 795, 796, 798, 799, 801, 803, 804, 806, 807, 809, 811,
		812, 814, 815, 817, 819, 820, 822, 824, 825, 827, 828, 830, 832, 833,
		835, 837, 838, 840, 842, 843, 845, 847, 848, 850, 852, 853, 855, 857,
		858, 860, 862, 863, 865, 867, 868, 870, 872, 873, 875, 877, 878, 880,
		882, 884, 885, 887, 889, 890, 892, 894, 896, 897, 899, 901, 902, 904,
		906, 908, 909, 911, 913, 915, 916, 918, 920, 922, 923, 925, 927, 929,
		930, 932, 934, 936, 938, 939, 941, 943, 945, 947, 948, 950, 952, 954,
		956, 957, 959, 961, 963, 965, 966, 968, 970, 972, 974, 976, 977, 979,
		981, 983, 985, 987, 988, 990, 992, 994, 996, 998, 1000, 1001, 1003,
		1005, 1007, 1009, 1011, 1013, 1015, 1016, 1018, 1020, 1022, 1024 };

#endif

/** @ingroup MASTER
 * AI4 の変わりに電源電圧を読み出して送信する。
 *
 * 2.5V 以上 (PWM=0%) 2.5V 以下 (PWM=50%), 2.3V以下 (PWM=100%)
 */
#undef USE_PWM4_AS_BAT_MONITOR
#ifdef USE_PWM4_AS_BAT_MONITOR
#warning "EXPERIMENTAL: USE_PWM4_AS_BAT_MONITOR"
#endif

/** @ingroup MASTER
 * 間欠モード時にゆっくり送信する
 */
#undef USE_SLOW_TX
#ifdef USE_SLOW_TX
#warning "EXPERIMENTAL: USE_SLOW_TX"
#endif
#endif /* EXPERIMENTAL_H_ */
